@@ -4,13 +4,17 @@ from collections import defaultdict
 
 from lxml import etree
 
+from oca_pre_commit_hooks import utils
+
 DFTL_MIN_PRIORITY = 99
 DFLT_DEPRECATED_TREE_ATTRS = ["colors", "fonts", "string"]
 
 
 class ChecksOdooModuleXML:
-    def __init__(self, manifest_datas, module_name):
+    def __init__(self, manifest_datas, module_name, enable, disable):
         self.module_name = module_name
+        self.enable = enable
+        self.disable = disable
         self.manifest_datas = manifest_datas
         self.checks_errors = defaultdict(list)
         for manifest_data in manifest_datas:
@@ -22,14 +26,13 @@ class ChecksOdooModuleXML:
                             "file_error": None,
                         }
                     )
-            except (FileNotFoundError, etree.XMLSyntaxError) as xml_err:  # pragma: no cover
+            except (FileNotFoundError, etree.XMLSyntaxError, UnicodeDecodeError) as xml_err:
                 manifest_data.update(
                     {
                         "node": etree.Element("__empty__"),
                         "file_error": xml_err,
                     }
                 )
-                self.checks_errors["xml_syntax_error"].append(f'{manifest_data["filename"]} {xml_err}')
 
     @staticmethod
     def _get_priority(view):
@@ -51,7 +54,7 @@ class ChecksOdooModuleXML:
         return bool(replaces)
 
     def check_xml_records(self):
-        """* Check xml_duplicate_record_id
+        """* Check xml-duplicate-record-id
 
         If a module has duplicated record_id AKA xml_ids
         file1.xml
@@ -59,7 +62,7 @@ class ChecksOdooModuleXML:
         file2.xml
             <record id="xmlid_name1"
 
-        * Check xml_duplicate_fields in all record nodes
+        * Check xml-duplicate-fields in all record nodes
             <record id="xmlid_name1"...
                 <field name="field_name1"...
                 <field name="field_name1"...
@@ -91,18 +94,16 @@ class ChecksOdooModuleXML:
                         )
                         xml_fields[field_key].append((manifest_data, field))
 
-                self.visit_xml_record(manifest_data, record)
-                self.visit_xml_record_view(manifest_data, record)
-                self.visit_xml_record_user(manifest_data, record)
-                self.visit_xml_record_filter(manifest_data, record)
+                for meth in utils.getattr_checks(self, self.enable, self.disable, "visit_xml_record"):
+                    meth(manifest_data, record)
 
         # xmlids_duplicated
         for xmlid_key, records in xmlids_section.items():
             if len(records) < 2:
                 continue
-            lines_str = ", ".join(f"{record[0]['filename']}:{record[1].sourceline}" for record in records[1:])
-            self.checks_errors["xml_duplicate_record_id"].append(
-                f"{records[0][0]['filename']}:{records[0][1].sourceline} "
+            lines_str = ", ".join(f"{record[0]['filename_short']}:{record[1].sourceline}" for record in records[1:])
+            self.checks_errors["xml-duplicate-record-id"].append(
+                f"{records[0][0]['filename_short']}:{records[0][1].sourceline} "
                 f'Duplicate xml record id "{xmlid_key}" in {lines_str}'
             )
 
@@ -111,13 +112,25 @@ class ChecksOdooModuleXML:
             if len(fields) < 2:
                 continue
             lines_str = ", ".join(f"{field[1].sourceline}" for field in fields[1:])
-            self.checks_errors["xml_duplicate_fields"].append(
-                f"{fields[0][0]['filename']}:{fields[0][1].sourceline} "
+            self.checks_errors["xml-duplicate-fields"].append(
+                f"{fields[0][0]['filename_short']}:{fields[0][1].sourceline} "
                 f'Duplicate xml field "{field_key[0]}" in lines {lines_str}'
             )
 
+    @utils.only_required_for_checks("xml-syntax-error")
+    def check_xml_syntax_error(self):
+        """* Check xml-syntax-error
+        Check syntax of XML files declared in the Odoo manifest"""
+        for manifest_data in self.manifest_datas:
+            if not manifest_data["file_error"]:
+                continue
+            self.checks_errors["xml-syntax-error"].append(
+                f'{manifest_data["filename_short"]}:1 {manifest_data["file_error"]}'
+            )
+
+    @utils.only_required_for_checks("xml-redundant-module-name")
     def visit_xml_record(self, manifest_data, record):
-        """* Check xml_redundant_module_name
+        """* Check xml-redundant-module-name
 
         If the module is called "module_a" and the xmlid is
         <record id="module_a.xmlid_name1" ...
@@ -130,20 +143,21 @@ class ChecksOdooModuleXML:
         xmlid_module, xmlid_name = record_id.split(".") if "." in record_id else ["", record_id]
         if xmlid_module == self.module_name:
             # TODO: Add autofix option
-            self.checks_errors["xml_redundant_module_name"].append(
-                f'{manifest_data["filename"]}:{record.sourceline} Redundant module'
+            self.checks_errors["xml-redundant-module-name"].append(
+                f'{manifest_data["filename_short"]}:{record.sourceline} Redundant module'
                 f' name <record id="{record_id}" '
                 f'better using only <record id="{xmlid_name}"'
             )
 
+    @utils.only_required_for_checks("xml-view-dangerous-replace-low-priority", "xml-deprecated-tree-attribute")
     def visit_xml_record_view(self, manifest_data, record):
-        """* Check xml_view_dangerous_replace_low_priority in ir.ui.view
+        """* Check xml-view-dangerous-replace-low-priority in ir.ui.view
 
             <field name="priority" eval="10"/>
             ...
                 <field name="name" position="replace"/>
 
-        * Check xml_deprecated_tree_attribute
+        * Check xml-deprecated-tree-attribute
           The tree-view declaration is using a deprecated attribute.
         """
         if record.get("model") != "ir.ui.view":
@@ -153,8 +167,8 @@ class ChecksOdooModuleXML:
         is_replaced_field = self._is_replaced_field(record)
         # TODO: Add self.config.min_priority instead of DFTL_MIN_PRIORITY
         if is_replaced_field and priority < DFTL_MIN_PRIORITY:
-            self.checks_errors["xml_view_dangerous_replace_low_priority"].append(
-                f'{manifest_data["filename"]}:{record.sourceline} '
+            self.checks_errors["xml-view-dangerous-replace-low-priority"].append(
+                f'{manifest_data["filename_short"]}:{record.sourceline} '
                 'Dangerous use of "replace" from view '
                 f"with priority {priority} < {DFTL_MIN_PRIORITY}"
             )
@@ -164,13 +178,14 @@ class ChecksOdooModuleXML:
         xpath = f".//tree[{'|'.join(f'@{a}' for a in deprecate_attrs)}]"
         for deprecate_attr_node in record.xpath(xpath):
             deprecate_attr_str = ",".join(set(deprecate_attr_node.attrib.keys()) & deprecate_attrs)
-            self.checks_errors["xml_deprecated_tree_attribute"].append(
-                f'{manifest_data["filename"]}:{deprecate_attr_node.sourceline} '
+            self.checks_errors["xml-deprecated-tree-attribute"].append(
+                f'{manifest_data["filename_short"]}:{deprecate_attr_node.sourceline} '
                 f'Deprecated "<tree {deprecate_attr_str}=..."'
             )
 
+    @utils.only_required_for_checks("xml-create-user-wo-reset-password")
     def visit_xml_record_user(self, manifest_data, record):
-        """* Check xml_create_user_wo_reset_password
+        """* Check xml-create-user-wo-reset-password
         records of user without context="{'no_reset_password': True}"
         This context avoid send email and mail log warning
         """
@@ -180,14 +195,15 @@ class ChecksOdooModuleXML:
         if record.xpath("field[@name='name'][1]") and "no_reset_password" not in (record.get("context") or ""):
             # if exists field="name" then is a new record
             # then should be context
-            self.checks_errors["xml_create_user_wo_reset_password"].append(
-                f'{manifest_data["filename"]}:{record.sourceline} '
+            self.checks_errors["xml-create-user-wo-reset-password"].append(
+                f'{manifest_data["filename_short"]}:{record.sourceline} '
                 "record res.users without "
                 "context=\"{'no_reset_password': True}\""
             )
 
+    @utils.only_required_for_checks("xml-dangerous-filter-wo-user")
     def visit_xml_record_filter(self, manifest_data, record):
-        """* Check xml_dangerous_filter_wo_user
+        """* Check xml-dangerous-filter-wo-user
         Check dangerous filter without a user assigned.
         """
         # xml_dangerous_filter_wo_user
@@ -197,12 +213,14 @@ class ChecksOdooModuleXML:
         # if exists field="name" then is a new record
         # then should be field="user_id" too
         if ir_filter_fields and len(ir_filter_fields) == 1:
-            self.checks_errors["xml_dangerous_filter_wo_user"].append(
-                f'{manifest_data["filename"]}:{record.sourceline} ' "Dangerous filter without explicit `user_id`"
+            self.checks_errors["xml-dangerous-filter-wo-user"].append(
+                f'{manifest_data["filename_short"]}:{record.sourceline} ' "Dangerous filter without explicit `user_id`"
             )
 
+    @utils.only_required_for_checks("xml-not-valid-char-link")
     def check_xml_not_valid_char_link(self):
-        """The resource in in src/href contains a not valid character"""
+        """* Check xml-not-valid-char-link
+        The resource in in src/href contains a not valid character."""
         for manifest_data in self.manifest_datas:
             for name, attr in (("link", "href"), ("script", "src")):
                 nodes = manifest_data["node"].xpath(f".//{name}[@{attr}]")
@@ -210,13 +228,15 @@ class ChecksOdooModuleXML:
                     resource = node.get(attr, "")
                     ext = os.path.splitext(os.path.basename(resource))[1]
                     if resource.startswith("/") and not re.search("^[.][a-zA-Z]+$", ext):
-                        self.checks_errors["xml_not_valid_char_link"].append(
-                            f'{manifest_data["filename"]}:{node.sourceline} '
+                        self.checks_errors["xml-not-valid-char-link"].append(
+                            f'{manifest_data["filename_short"]}:{node.sourceline} '
                             f"The resource in in src/href contains a not valid character"
                         )
 
+    @utils.only_required_for_checks("xml-dangerous-qweb-replace-low-priority")
     def check_xml_dangerous_qweb_replace_low_priority(self):
-        """Check dangerous qweb view defined with low priority"""
+        """* Check xml-dangerous-qweb-replace-low-priority
+        Dangerous qweb view defined with low priority"""
         for manifest_data in self.manifest_datas:
             for template in manifest_data["node"].xpath("/odoo//template|/openerp//template"):
                 try:
@@ -226,14 +246,16 @@ class ChecksOdooModuleXML:
                 for child in template.iterchildren():
                     # TODO: Add self.config.min_priority instead of DFTL_MIN_PRIORITY
                     if child.get("position") == "replace" and priority < DFTL_MIN_PRIORITY:
-                        self.checks_errors["xml_dangerous_qweb_replace_low_priority"].append(
-                            f'{manifest_data["filename"]}:{template.sourceline} '
+                        self.checks_errors["xml-dangerous-qweb-replace-low-priority"].append(
+                            f'{manifest_data["filename_short"]}:{template.sourceline} '
                             'Dangerous use of "replace" from view '
                             f"with priority {priority} < {DFTL_MIN_PRIORITY}"
                         )
 
+    @utils.only_required_for_checks("xml-deprecated-data-node")
     def check_xml_deprecated_data_node(self):
-        """Check deprecated <data> node inside <odoo> xml node"""
+        """* Check xml-deprecated-data-node
+        Deprecated <data> node inside <odoo> xml node"""
         for manifest_data in self.manifest_datas:
             for odoo_node in manifest_data["node"].xpath("/odoo|/openerp"):
                 children_count = 0
@@ -245,23 +267,27 @@ class ChecksOdooModuleXML:
                 #     import pdb;pdb.set_trace()
                 if children_count == 1 and len(odoo_node.xpath("./data")) == 1:
                     # TODO: Add autofix option
-                    self.checks_errors["xml_deprecated_data_node"].append(
-                        f'{manifest_data["filename"]}:{odoo_node.sourceline} '
-                        'Use <odoo> instead of <odoo><data> or use <odoo noupdate="1"> '
-                        'instead of <odoo><data noupdate="1">'
+                    self.checks_errors["xml-deprecated-data-node"].append(
+                        f'{manifest_data["filename_short"]}:{odoo_node.sourceline} '
+                        'Use `<odoo>` instead of `<odoo><data>` or use `<odoo noupdate="1">` '
+                        'instead of `<odoo><data noupdate="1">`'
                     )
 
+    @utils.only_required_for_checks("xml-deprecated-openerp-xml-node")
     def check_xml_deprecated_openerp_node(self):
-        """Check deprecated <openerp> xml node"""
+        """* Check xml-deprecated-openerp-xml-node
+        deprecated <openerp> xml node"""
         for manifest_data in self.manifest_datas:
             for openerp_node in manifest_data["node"].xpath("/openerp"):
                 # TODO: Add autofix option
-                self.checks_errors["xml_deprecated_openerp_xml_node"].append(
-                    f'{manifest_data["filename"]}:{openerp_node.sourceline} ' "Deprecated <openerp> xml node"
+                self.checks_errors["xml-deprecated-openerp-xml-node"].append(
+                    f'{manifest_data["filename_short"]}:{openerp_node.sourceline} ' "Deprecated <openerp> xml node"
                 )
 
+    @utils.only_required_for_checks("xml-deprecated-qweb-directive")
     def check_xml_deprecated_qweb_directive(self):
-        """Check for use of deprecated QWeb directives t-*-options"""
+        """* Check xml-deprecated-qweb-directive
+        for use of deprecated QWeb directives t-*-options"""
         deprecated_directives = {
             "t-esc-options",
             "t-field-options",
@@ -273,7 +299,7 @@ class ChecksOdooModuleXML:
         for manifest_data in self.manifest_datas:
             for node in manifest_data["node"].xpath(xpath):
                 directive_str = ", ".join(set(node.attrib) & deprecated_directives)
-                self.checks_errors["xml_deprecated_qweb_directive"].append(
-                    f'{manifest_data["filename"]}:{node.sourceline} '
+                self.checks_errors["xml-deprecated-qweb-directive"].append(
+                    f'{manifest_data["filename_short"]}:{node.sourceline} '
                     f'Deprecated QWeb directive "{directive_str}". Use "t-options" instead'
                 )
